#-------------------------------------------------------------.
# ポーズを追加.
# @param[in]  f_location      カレントパス (string).
# @param[in]  pattern_name    対象のパターン名.
# @param[in]  pose_name       ポーズ名.
# @param[in]  ignore_bone_names 無視するボーン名（コンマ区切り）.
#-------------------------------------------------------------.
import urllib
import os
import os.path
import shutil

PATTERN_FILE_NAME = 'pattern_data.txt'
TEMP_FILE_NAME    = 'pattern_data_tmp.txt'

# 格納時に無視するボーン名.
ignoreBonesList = []
if len(ignore_bone_names) > 0:
	ignoreBonesList = ignore_bone_names.split(',')
	for i in range(len(ignoreBonesList)):
		ignoreBonesList[i] = ignoreBonesList[i].strip()

# --------------------------------------------------------------.
# --------------------------------------------------------------.
# 指定のボーンに割り当てられているカレントのオフセットと回転を.
# ボーン名をキーとしてファイルに記録.
# --------------------------------------------------------------.
def writeBoneOffsetRotation (f, bone_shape):
	
	if bone_shape.has_motion == False:
		return

	# ボーンかボールジョイントの場合.
	if bone_shape.type == 2 and (bone_shape.part_type == 10 or bone_shape.part_type == 16):
		jointOffset   = bone_shape.bone_joint.offset
		jointRotation = bone_shape.bone_joint.rotation

		wStr = '[' + bone_shape.name + '] ' + str(jointOffset[0]) + ',' + str(jointOffset[1]) + ',' + str(jointOffset[2]) + '  ,' + str(jointRotation[0]) + ',' + str(jointRotation[1]) + ',' + str(jointRotation[2]) + ',' + str(jointRotation[3])
		
		f.writelines(wStr)
		f.writelines('\n')

# --------------------------------------------------------------.
# ボーンをたどり、ファイルにオフセットと回転情報を保存していく.
# --------------------------------------------------------------.
def writeBonesData (f, bone_shape):
	name = bone_shape.name

	# 無視するボーンを調べる.
	findF = False
	for i in range(len(ignoreBonesList)):
		if name.find(ignoreBonesList[i]) >= 0:
			findF = True
			break

	if findF:
		return

	writeBoneOffsetRotation(f, bone_shape)

	if bone_shape.has_son:
		s = bone_shape.son
		while s.has_bro:
			s = s.bro
			writeBonesData(f, s)

# ----------------------------------------.
# ポーズ情報を書き出し.
# ----------------------------------------.
def writePoseData(wf):
	wf.writelines('## ' + pose_name + '\n')
	shape = xshade.scene().active_shape()
	writeBonesData(wf, shape)

# --------------------------------------------------------------.

targetFile = urllib.unquote(f_location) + '/datas/' + PATTERN_FILE_NAME
tempFile   = urllib.unquote(f_location) + '/datas/' + TEMP_FILE_NAME

if os.path.exists(targetFile) == True:
	f  = open(targetFile, 'r')
	wf = open(tempFile, 'w')

	aState = 0
	lineStr = f.readline()
	while lineStr:
		lineStr = lineStr.strip()		# 行頭、行末の改行や空白を削除.

		if len(lineStr) != 0:
			if len(lineStr) > 2:
				if lineStr[0] == '#' and lineStr[1] != '#':
					patternN = lineStr[1:]
					patternN = patternN.strip()

					if aState == 0 and patternN == pattern_name:
						aState += 1
					else:
						if aState == 1:
							writePoseData(wf)
							aState += 1

		wf.writelines(lineStr + '\n')
		lineStr = f.readline()

	if aState == 1:
		writePoseData(wf)
		aState += 1

	f.close()
	wf.close()

	shutil.copyfile(tempFile, targetFile)
	os.remove(tempFile)

result = ''
