#-------------------------------------------------------------.
# スライダの範囲でカレントシーケンスでのポーズを反映.
# @param[in]  f_location      カレントパス (string).
# @param[in]  pattern_name    対象のパターン名.
# @param[in]  slider_value    0.0 - 100.0の位置.
#-------------------------------------------------------------.
import urllib
import os
import os.path
import shutil

PATTERN_FILE_NAME = 'pattern_data.txt'

slider_value = float(slider_value) * 0.01

# --------------------------------------------------------------.
# 各ポーズの情報格納クラス.
# --------------------------------------------------------------.
class KeyframeData:
	name          = ''
	jointOffset   = [0.0, 0.0, 0.0]
	jointRotation = [1.0, 0.0, 0.0, 0.0]

class PoseData:
	poseName = ''
	keyframeList = []		# keyframeDataクラスをリスト格納する.

# --------------------------------------------------------------.
# テスト出力.
# --------------------------------------------------------------.
def outputPatternData(patternDataList):
	for i in range(len(patternDataList)):
		poseD = patternDataList[i]
		print 'poseName : ' + poseD.poseName
		
		for j in range(len(poseD.keyframeList)):
			keyD = poseD.keyframeList[j]
			print '   [' + keyD.name + '] offset : ' + str(keyD.jointOffset) + ' rotation : ' + str(keyD.jointRotation)

# --------------------------------------------------------------.
# ボーン情報を格納.
# --------------------------------------------------------------.
def storeBoneInfo(patternDataList, lineStr):
	if len(lineStr) == 0 or len(patternDataList) == 0:
		return
	if lineStr[0] == '#':
		return

	poseD = patternDataList[len(patternDataList) - 1]
	
	# [boneA] 0.0,0.0,0.0  ,1.0,0.0,0.0,0.0
	# のように格納されているボーン情報を、ボーン名/Offset/Rotationに分解して格納.
	st_index  = lineStr.find('[')
	end_index = lineStr.find(']')
	if st_index < 0 or end_index < 0:
		return

	# 対象ボーン名.
	boneName = lineStr[(st_index+1):end_index]

	# 数値部をコンマで分割.
	strV = lineStr[(end_index + 1):]
	numList = strV.split(',')
	numListCou = len(numList)
	if numListCou != 7:
		return

	# 文字列をfloatに変換.
	for i in range(numListCou):
		numList[i] = float(numList[i])

	poseD.keyframeList.append(KeyframeData())
	keyframeD = poseD.keyframeList[len(poseD.keyframeList) - 1]
	keyframeD.name          = boneName
	keyframeD.jointOffset   = [numList[0], numList[1], numList[2]]
	keyframeD.jointRotation = [numList[3], numList[4], numList[5], numList[6]]

# --------------------------------------------------------------.
# boneShapeのボーンに対して、sliderVの位置でのoffset/rotationを指定.
# --------------------------------------------------------------.
def setJointInfo(boneShape, keyframeDList, sliderV):
	poseCou = len(keyframeDList)
	if poseCou == 0 or boneShape == None:
		return

	# ボーンまたはボールジョイントかチェック.
	if boneShape.type != 2 or (boneShape.part_type != 10 and boneShape.part_type != 16):
		return

	# 要素が1つしかない場合、または sliderV = 0.0 の場合.
	if poseCou == 1 or isZero(sliderV):
		keyframeD = keyframeDList[0]
		boneShape.bone_joint.offset = [keyframeD.jointOffset[0], keyframeD.jointOffset[1], keyframeD.jointOffset[2]]
		boneShape.bone_joint.rotation = [keyframeD.jointRotation[0], keyframeD.jointRotation[1], keyframeD.jointRotation[2], keyframeD.jointRotation[3]]
		return

	# sliderV = 1.0 の場合.
	if isZero(sliderV - 1.0):
		keyframeD = keyframeDList[poseCou - 1]
		boneShape.bone_joint.offset = [keyframeD.jointOffset[0], keyframeD.jointOffset[1], keyframeD.jointOffset[2]]
		boneShape.bone_joint.rotation = [keyframeD.jointRotation[0], keyframeD.jointRotation[1], keyframeD.jointRotation[2], keyframeD.jointRotation[3]]
		return
	
	dd = 1.0 / float(poseCou - 1)
	posV = 0.0
	for i in range(poseCou - 1):
		stPos  = posV
		endPos = posV + dd
		if sliderV >= stPos and sliderV <= endPos:
			v = (sliderV - stPos) / (endPos - stPos)
			keyframeD1 = keyframeDList[i]
			keyframeD2 = keyframeDList[i + 1]

			p1 = set_vec3(keyframeD1.jointOffset)
			p2 = set_vec3(keyframeD2.jointOffset)
			offset = p1 * (1.0 - v) + p2 * v

			q1 = set_quaternion(keyframeD1.jointRotation)
			q2 = set_quaternion(keyframeD2.jointRotation)
			rotation = q1 * (1.0 - v) + q2 * v

			boneShape.bone_joint.offset   = [offset[0], offset[1], offset[2]]
			boneShape.bone_joint.rotation = [rotation[0], rotation[1], rotation[2], rotation[3]]
			break

		posV += dd

# --------------------------------------------------------------.
# カレント形状より、指定名のボーンを検索.
# --------------------------------------------------------------.
def findShape(shape, targetName):
	if shape.name == targetName:
		return shape

	# HumanIKのボーン名を考慮して検索.
	# xxxx_Hips などのような場合に、ハイフン後の名称で比較.
	p1 = shape.name.find('_')
	p2 = targetName.find('_')
	if p1 >= 0 and p2 >= 0:
		if shape.name[p1+1:] == targetName[p2+1:]:
			return shape

	retShape = None
	if shape.has_son:
		s = shape.son
		while s.has_bro:
			s = s.bro
			retShape = findShape(s, targetName)
			if retShape != None:
				break

	return retShape

# --------------------------------------------------------------.

targetFile = urllib.unquote(f_location) + '/datas/' + PATTERN_FILE_NAME

# --------------------------------------------------------------------.
# pattern_name に対応するポーズ情報を配列 patternDataList に格納.
# --------------------------------------------------------------------.
patternDataList = []		# PoseDataクラスをリスト格納する.

if os.path.exists(targetFile) == True:
	f  = open(targetFile, 'r')

	aState = 0
	lineStr = f.readline()
	while lineStr:
		lineStr = lineStr.strip()		# 行頭、行末の改行や空白を削除.

		if len(lineStr) != 0:
			if len(lineStr) > 2:
				# パターン名が見つかった場合.
				if lineStr[0] == '#' and lineStr[1] != '#':
					patternN = lineStr[1:]
					patternN = patternN.strip()
					if aState == 0 and patternN == pattern_name:
						aState += 1
					else:
						if aState > 0:
							aState += 1

				# ポーズ名が見つかった場合.
				if lineStr[0] == '#' and lineStr[1] == '#':
					poseN = lineStr[2:]
					poseN = poseN.strip()

					if aState == 1:
						patternDataList.append(PoseData())
						patternDataList[len(patternDataList)-1].poseName = poseN
						patternDataList[len(patternDataList)-1].keyframeList = []

				# ボーン情報を格納.
				if aState == 1 and lineStr[0] != '#':
					storeBoneInfo(patternDataList, lineStr)

		lineStr = f.readline()

	f.close()

	# test...
	#outputPatternData(patternDataList)

# --------------------------------------------------------------------.
# パターン情報より、slider_valueの位置でoffset/rotationをボーンに割り当てていく.
# --------------------------------------------------------------------.
if len(patternDataList) >= 1:
	active_shape = xshade.scene().active_shape()

	# ボーンリストを取得.
	boneNameList = []
	poseD = patternDataList[0]
	for i in range(len(poseD.keyframeList)):
		boneNameList.append(poseD.keyframeList[i].name)

	boneCou = len(boneNameList)

	# ボーンごとにoffset/rotationの配列を格納.
	for bLoop in range(boneCou):
		keyframeDList = []

		for i in range(len(patternDataList)):
			poseD = patternDataList[i]
			for j in range(len(poseD.keyframeList)):
				keyframeD = poseD.keyframeList[j]
				if keyframeD.name == boneNameList[bLoop]:
					keyframeDList.append(keyframeD)
					break

		# boneNameList[bLoop] のボーンに対して、slider_valueの位置でのoffset/rotationを指定.
		boneShape = findShape(active_shape, boneNameList[bLoop])
		setJointInfo(boneShape, keyframeDList, slider_value)

	# 以下で強制的に更新をかけないと、更新されない場合がある.
	active_shape.update_skin_bindings()
	active_shape.update()

	# 強制的に再描画.
	xshade.scene().force_update()

result = ''
